/* Copyright (c) 2015-present, salesforce.com, inc. All rights reserved
  Licensed under BSD 3-Clause - see LICENSE.txt or git.io/sfdc-license */

import { fragmentFrom } from 'elix/src/core/htmlLiterals';
import { firstRender, hasDynamicTemplate, render, setState, state, template } from 'elix/src/base/internal';
import { standardBooleanAttributes } from 'elix/src/core/AttributeMarshallingMixin';
import SdsInputCheckbox from '@salesforce-ux/sds-input-checkbox/input-checkbox';
import { delegateAttribute, reflectAttribute } from '@salesforce-ux/sds-utils';

import html from './input-checkbox.html';
import css from './input-checkbox.module.css';

/**
 * > FOR TESTING ONLY <
 *
 * This component represents the native shadow DOM half of mixed-mode. Mixed-mode
 * currently requires markup and CSS to be the same in both modes.
 */

/**
 * Extend Elix's standard boolean attributes definition
 */
Object.assign(standardBooleanAttributes, {
  invalid: true,
  fieldlevelhelp: true,
});
export class SldsInputCheckbox extends SdsInputCheckbox {
  get [hasDynamicTemplate]() {
    return true;
  }

  get invalid() {
    return this[state].invalid;
  }

  set invalid(invalid) {
    this[setState]({ invalid });
  }

  get required() {
    return this[state].required;
  }

  set required(required) {
    this[setState]({ required });
  }

  get variant() {
    return this[state].variant;
  }

  set variant(variant) {
    this[setState]({ variant });
  }

  /**
   * @TODO: This should be camel case (fieldLevelHelp) but having some trouble with Elix and
   * transforming kebab case boolean attributes from HTML.
   */
  get fieldlevelhelp() {
    return this[state].fieldlevelhelp;
  }

  set fieldlevelhelp(fieldlevelhelp) {
    this[setState]({ fieldlevelhelp });
  }

  /**
   * Handle internal 'change' event triggered by checkbox
   */
  _handleChange(checkbox) {
    super._handleChange(checkbox);
    const inputCheckbox = this.shadowRoot.querySelector('[part~="input-checkbox"]');

    /**
     * Sync our states with the IDL properties of the checkbox
     */
    this[setState]({ invalid: !checkbox.validity.valid });

    this._handleInvalid(inputCheckbox);
  }

  /**
   * Handle invalid state of checkbox
   *
   * This is a mock validation implementation for the sole purpose of mimicking
   * LBC validation behavior and provding styling examples.
   */
  _handleInvalid(inputCheckbox) {
    const invalid = this[state].invalid;
    const helpText = this.shadowRoot.querySelector('[part~="help-text"]');
    const helpTextEl = `<div id="help-message" role="alert" part="help-text">Complete this field.</div>`;

    if (invalid && !helpText) {
      inputCheckbox.insertAdjacentHTML('afterend', helpTextEl);
    }

    if (!invalid && helpText) {
      helpText.remove();
    }
  }

  /**
   * Handle required state of checkbox
   *
   * This is a mock implementation for the sole purpose of mimicking
   * LBC required behavior and provding styling examples.
   */
  _handleRequired(checkbox) {
    const required = this[state].required;
    const requiredMarker = this.shadowRoot.querySelector('[title="required"]');
    const requiredMarkerEl = `<abbr class="slds-required" title="required">*</abbr>`;
    const label = this.shadowRoot.querySelector('[part~="label"]');
    const { variant } = this[state];

    if (required && !requiredMarker) {
      variant === 'label-inline' || variant === 'label-stacked'
        ? label.insertAdjacentHTML('beforebegin', requiredMarkerEl)
        : checkbox.insertAdjacentHTML('beforebegin', requiredMarkerEl);
    }

    if (!required && requiredMarker) {
      required.remove();
    }
  }

  [render](changed) {
    super[render](changed);
    const { id, invalid, variant } = this[state];
    const inputCheckbox = this.shadowRoot.querySelector('[part~="input-checkbox"]');
    const checkbox = this.shadowRoot.querySelector('[part~="checkbox"]');
    const labelContainer = this.shadowRoot.querySelector('[part~="label-container"]');
    const label = this.shadowRoot.querySelector('[part~="label"]');

    if (this[firstRender] && invalid) {
      this._handleInvalid(inputCheckbox);
    }

    if (changed.invalid) {
      reflectAttribute(this, 'invalid', invalid);
    }

    if (changed.required) {
      this._handleRequired(checkbox);
    }

    if (changed.variant) {
      variant === 'label-hidden'
        ? label.classList.add('slds-assistive-text')
        : label.classList.remove('slds-assistive-text');
    }

    /**
     * Label element is the container, part="label-container", instead of a
     * single label element with a string as implemented in SDS.
     */
    if (changed.id) {
      if (id) {
        delegateAttribute(this, checkbox, 'id', id);
        labelContainer.setAttribute('for', id);
        label.removeAttribute('for');
      }
    }
  }

  createLabelVariant() {
    const { variant } = this[state];
    const dynamicTemplate =
      variant === 'label-inline' || variant === 'label-stacked'
        ? `
            <label part="label-container">
              <span part="label">I'm a label</span>
            </label>
          `
        : `
            <label part="label-container">
              <span part="indicator"></span>
              <span part="label">I'm a label</span>
            </label>
        `;
    return dynamicTemplate;
  }

  createIndicatorVariant() {
    const dynamicTemplate = `<span part="indicator"></span>`;

    return dynamicTemplate;
  }

  createInlineHelp() {
    const dynamicTemplate = `<slds-help-text icon-name="utility:info" part="inline-help"></slds-help-text>`;

    return dynamicTemplate;
  }

  /**
   * Blow away SDS markup, we need LBC markup structure here. But we decorate with
   * `part` to align with the sds-input-checkbox's spec. We also don't completely
   * remove the SDS template since we still want a dynamic link to the styles.
   */
  get [template]() {
    const result = super[template];
    result.content.querySelector('[part~="input-checkbox"]').remove();
    result.content.append(fragmentFrom.html`
      <style>
        ${css}
      </style>
      ${html}
    `);

    // Populate markup dependent on variant
    const { fieldlevelhelp, variant } = this[state];
    const container = result.content.querySelector('[part~="input-checkbox"]');
    const checkbox = result.content.querySelector('[part~="checkbox"]');
    const labelVariant = this.createLabelVariant();
    const indicatorVariant = this.createIndicatorVariant();

    if (variant === 'label-inline' || variant === 'label-stacked') {
      container.insertAdjacentHTML('beforebegin', labelVariant);
      checkbox.insertAdjacentHTML('afterend', indicatorVariant);
    } else {
      checkbox.insertAdjacentHTML('afterend', labelVariant);
    }

    // Populate Inline Help
    const labelContainer = result.content.querySelector('[part~="label-container"]');
    const inlineHelp = this.createInlineHelp();
    fieldlevelhelp && labelContainer.insertAdjacentHTML('afterend', inlineHelp);

    return result;
  }
}
